from menus.templatetags.menu_tags import ShowMenu
from classytags.arguments import IntegerArgument, Argument, StringArgument
from classytags.core import Options
from django import template
from ..models import CMSNamedMenu
from django.core.exceptions import ObjectDoesNotExist
import logging
from menus.menu_pool import menu_pool

logger = logging.getLogger(__name__)

register = template.Library()


class ShowMultipleMenu(ShowMenu):
    name = 'show_named_menu'

    options = Options(
        StringArgument('menu_name', required=True),
        IntegerArgument('from_level', default=0, required=False),
        IntegerArgument('to_level', default=100, required=False),
        IntegerArgument('extra_inactive', default=0, required=False),
        IntegerArgument('extra_active', default=1000, required=False),
        StringArgument('template', default='menu/menu.html', required=False),
        StringArgument('namespace', default=None, required=False),
        StringArgument('root_id', default=None, required=False),
        Argument('next_page', default=None, required=False),
    )

    def get_context(self, context, **kwargs):

        menu_name = kwargs.pop('menu_name')

        context.update({'children': [],
                        'template': kwargs.get('template'),
                        'from_level': kwargs.get('from_level'),
                        'to_level': kwargs.get('to_level'),
                        'extra_inactive': kwargs.get('extra_inactive'),
                        'extra_active': kwargs.get('extra_active'),
                        'namespace': kwargs.get('namespace')
                        })

        try:
            named_menu = CMSNamedMenu.objects.get(name__iexact=menu_name).pages
        except ObjectDoesNotExist:
            logging.warn("Named CMS Menu %s not found" % menu_name)
            return context

        nodes = menu_pool.get_nodes(
            context['request'], kwargs['namespace'], kwargs['root_id'])

        context.update({
            'children': self.arrange_nodes(nodes, named_menu)
        })

        return context

    def arrange_nodes(self, node_list, node_config):

        arranged_nodes = []

        for item in node_config:

            arranged_nodes.append(self.create_node(item, node_list))

        return arranged_nodes

    def create_node(self, item, node_list):

        item_node = self.get_node_by_id(item['id'], node_list)

        for child_item in item.get('children', []):

            child_node = self.get_node_by_id(child_item['id'], node_list)

            item_node.children.append(child_node)

        return item_node

    def get_node_by_id(self, id, nodes):

        for node in nodes:

            if node.id == id:
                return node

        return


register.tag(ShowMultipleMenu)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CMSNamedMenu'
        db.create_table(u'cms_named_menus_cmsnamedmenu', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique_with=(), max_length=50, populate_from='name')),
            ('pages', self.gf('jsonfield.fields.JSONField')(default=[], null=True, blank=True)),
        ))
        db.send_create_signal(u'cms_named_menus', ['CMSNamedMenu'])


    def backwards(self, orm):
        # Deleting model 'CMSNamedMenu'
        db.delete_table(u'cms_named_menus_cmsnamedmenu')


    models = {
        u'cms_named_menus.cmsnamedmenu': {
            'Meta': {'object_name': 'CMSNamedMenu'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'pages': ('jsonfield.fields.JSONField', [], {'default': '[]', 'null': 'True', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': "'name'"})
        }
    }

    complete_apps = ['cms_named_menus']


from django.contrib import admin
from .models import CMSNamedMenu
import json
from django.conf import settings
from menus.menu_pool import menu_pool


class CMSNamedMenuAdmin(admin.ModelAdmin):
    change_form_template = 'cms_named_menus/change_form.html'

    readoly_fields = ('pages_json',)

    def change_view(self, request, object_id, form_url='', extra_context={}):

        menu_pages = CMSNamedMenu.objects.get(id=object_id).pages

        extra_context = {
            'menu_pages': json.dumps(menu_pages),
            'available_pages': self.serialize_navigation(request),
            'debug': settings.DEBUG,
        }

        return super(CMSNamedMenuAdmin, self).change_view(request,
                                                          object_id,
                                                          form_url,
                                                          extra_context)

    def serialize_navigation(self, request):

        nodes = menu_pool.get_nodes(request)

        cleaned = []

        for node in nodes:
            node.children = None
            node.parent = None
            cleaned.append(node.__dict__)

        return json.dumps(cleaned)


admin.site.register(CMSNamedMenu, CMSNamedMenuAdmin)



#/usr/bin/env python
import codecs
import os
from setuptools import setup, find_packages

read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()

setup(
    name='django-cms-named-menus',
    version='0.1.4',
    description='Allows you to add named menus like Wordpress',
    long_description=read(os.path.join(os.path.dirname(__file__), 'README.rst')),
    author='Ryan Bagwell',
    author_email='ryan@ryanbagwell.com',
    license='BSD',
    url='https://github.com/ryanbagwell/django-cms-named-menus/',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'Django>=1.6',
        'django-cms>=3.0',
        'jsonfield>=1.0.0',
    ],
)



from django.db import models
from autoslug.fields import AutoSlugField
from jsonfield import JSONField
import collections


class CMSNamedMenu(models.Model):
    name = models.CharField(max_length=255)
    slug = AutoSlugField(always_update=True, populate_from='name')
    pages = JSONField(blank=True, null=True,
                      load_kwargs={
                          'object_pairs_hook': collections.OrderedDict
                      },
                      default=[])

    def __unicode__(self):
        return self.name
